﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

public class GameMaster : MonoBehaviour
{
  DeserializedLevelsLoader levelLoader;
  public PlayerControl[] playerPrefabs;

  public GameObject gameUIPanel;
  public GameObject gameOverPanel;
  public GameObject menuBloomPanel;
  public GameObject menuPanel;
  public Text rankingText;
  private GameStateTracker gameStateTracker;

  private GameObject deleteAllChildrenAtRestart;

  void Start()
  {
    deleteAllChildrenAtRestart = GameObject.Find("ObjectsToDeleteAtRestart");
    gameStateTracker = GetComponent<GameStateTracker>();
    levelLoader = new DeserializedLevelsLoader();
    levelLoader.load();
  }

  public void loadLevel(int numberOfPlayers, int level = -1)
  {
    if (level < 0)
      level = Random.Range(0, levelLoader.getLevelCount());
    levelLoader.loadLevel(level);

    destroyDynamicObjects();
    spawnPlayers(numberOfPlayers);
  }


  
  private void deleteAllLevelObjects()
  {
    GameObject level = GameObject.Find("XmlItems");
    Destroy(level);
    destroyDynamicObjects();
  }

  //deletes all objects spawned during the game:
  private void destroyDynamicObjects()
  {
    foreach (Transform child in deleteAllChildrenAtRestart.transform)
    {
      if (child != deleteAllChildrenAtRestart.gameObject)
      {
        Destroy(child.gameObject, 0f);
      }
    }
  }

  private void spawnPlayers(int count)
  {
    Debug.Log("spawning " + count + " players");

    GameObject[] startPositions = GameObject.FindGameObjectsWithTag("Start_Position");
    if (count > startPositions.Length)
    {
      count = startPositions.Length;
      Debug.LogError("map is capped at " + count + " players");
    }

    shuffle(startPositions);

    for (int i = 0; i < count; i++)
    {
      PlayerControl player = (PlayerControl)Object.Instantiate(
          playerPrefabs[i % playerPrefabs.Length],
          startPositions[i].transform.position,
          Quaternion.identity);

      player.transform.parent = deleteAllChildrenAtRestart.transform;
      player.init(i + 1);
    }
  }

  private static void shuffle<T>(T[] array)
  {
    int i = array.Length;
    while (i > 1)
    {
      int k = Random.Range(0, i--);
      T t = array[k];
      array[k] = array[i];
      array[i] = t;
    }
  }

  public void gameOver(List<string> ranking)
  {
    gameOverPanel.SetActive(true);

    Debug.Log(ranking.Count);
    if (ranking.Count > 0)
    {
      rankingText.text = "1st: " + ranking[0];
    }
    if (ranking.Count > 1)
    {
      rankingText.text += "\n2nd: " + ranking[1];
    }
    if (ranking.Count > 2)
    {
      rankingText.text += "\n3rd: " + ranking[2];
    }
    if (ranking.Count > 3)
    {
      rankingText.text += "\n4th: " + ranking[3];
    }
  }


  private void restartGame()
  {
    gameStateTracker.reset();

    loadLevel(gameStateTracker.getNumberOfPlayers()); //loads a new level randomly
    rankingText.text = "";
    gameOverPanel.SetActive(false);
  }

  private void goToMenu()
  {
    rankingText.text = "";
    deleteAllLevelObjects();

    menuBloomPanel.SetActive(true);
    menuPanel.SetActive(true);
    menuPanel.GetComponent<Menu>().reset();
    gameOverPanel.SetActive(false);
    gameUIPanel.SetActive(false);
  }

  void Update()
  {
    if (gameOverPanel.activeSelf)
    {
      if (Input.GetKeyDown(KeyCode.Space))
      {
        restartGame();
      }
      else if (Input.GetKeyDown(KeyCode.Return))
      {
        goToMenu();
      }
    }
  }
}
